/*
  Copyright (c) 2009, Matyas Bene <notabenem@hotmail.com>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer. 
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Neither the name of ReichlSoft nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "DB_Backup.h"
#include "Options.h"
#include "Utils.h"
#include "resource.h"
#include <tchar.h>
#include <stdio.h>
#include <shellapi.h>
#include <malloc.h>
#include <assert.h>


#ifdef _DEBUG
#define VERIFY(f)          assert(f)
#else
#define VERIFY(f)          ((void)(f))
#endif

#define KPM_SAVEDB_NOW 0xFF000001

static KP_MENU_ITEM g_menuItems[DB_BACKUP_MENU_ITEM_COUNT];

KP_IMPL_STDREFIMPL(CDB_BackupImpl)

// TODO: Delete later
IKpPlugin * myClass;

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	// Create the singleton holding global variables
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	DBBckp_S->hInstDLL=hinstDLL;
	UNREFERENCED_PARAMETER(fdwReason);
	UNREFERENCED_PARAMETER(lpvReserved);

	return TRUE;
}

KP_EXPORT HRESULT KP_API KP_I_INITIALIZELIB_DECL(IKpUnknown* pAPI)
{
	UNREFERENCED_PARAMETER(pAPI);
	return S_OK;
}

KP_EXPORT HRESULT KP_API KP_I_RELEASELIB_DECL(IKpUnknown* pAPI)
{
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	SAFE_RELEASE(DBBckp_S->g_pAPI);
	#ifdef _MYDEBUG
	fclose(DBBckp_S->fout);
	#endif
	
	UNREFERENCED_PARAMETER(pAPI);
	return S_OK;
}

KP_EXPORT HRESULT KP_API KP_I_CREATEINSTANCE_DECL(REFIID riid, void** ppvObject,
   IKpUnknown* pAPI)
{
	*ppvObject = NULL;
	if(pAPI == NULL) return E_UNEXPECTED; // Require API

	if((riid == IID_IKpUnknown) || (riid == IID_IKpPlugin))
	{
		DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
		if(pAPI->QueryInterface(IID_IKpAPI, (void**)&DBBckp_S->g_pAPI) != S_OK)
		 return E_FAIL;

		IKpPlugin* pImpl = myClass = new CDB_BackupImpl();
		*ppvObject = ((riid == IID_IKpPlugin) ? pImpl : (IKpUnknown*)pImpl);
		return S_OK;
	}

	return E_NOINTERFACE;
}

CDB_BackupImpl::CDB_BackupImpl()
{
	KP_IMPL_CONSTRUCT;

	// Initialize the translation engine
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();

	InitTranslationEngine(DBBckp_S->g_pAPI->GetTranslationName());

	// Set up the menus
	ZeroMemory(&g_menuItems, sizeof(KP_MENU_ITEM) * DB_BACKUP_MENU_ITEM_COUNT);

	g_menuItems[0].lpCommandString = _TRN(S_PLGNAME);	//_T("DB Backup Plugin");
	g_menuItems[0].dwFlags = KPMIF_POPUP_START;
	g_menuItems[1].lpCommandString = _TRN(S_MNU1);		//_T("&Backup DB NOW!");
	g_menuItems[1].dwIcon = 36;
	g_menuItems[1].dwFlags = KPMIF_DISABLED;
	g_menuItems[2].lpCommandString = _T("");
	g_menuItems[3].lpCommandString = _TRN(S_MNU3);		//_T("Automatically Backup DBs on &Save");
	g_menuItems[3].dwFlags = KPMIF_CHECKBOX;
	// Initialize the value a bit later (at the point when we will already know
	// the command line parameters

	g_menuItems[4].lpCommandString = _TRN(S_MNU4);		//_T("Automatically Backup DBs on &Open");
	g_menuItems[4].dwFlags = KPMIF_CHECKBOX;
	g_menuItems[4].dwState = KPMIS_UNCHECKED;
	g_menuItems[5].lpCommandString = _TRN(S_MNU5);
	g_menuItems[5].dwIcon = 5;
	g_menuItems[6].lpCommandString = _T("");
	g_menuItems[7].lpCommandString = _TRN(S_MNU7);		//_T("&About DB Backup Plugin...");
	g_menuItems[7].dwIcon = 23;
	g_menuItems[8].lpCommandString = _T("");
	g_menuItems[8].dwFlags = KPMIF_POPUP_END;
}

STDMETHODIMP CDB_BackupImpl::QueryInterface(REFIID riid, void** ppvObject)
{
   KP_SUPPORT_INTERFACE(IID_IKpUnknown, IKpUnknown);
   KP_SUPPORT_INTERFACE(IID_IKpPlugin, IKpPlugin);

   *ppvObject = NULL;
   return E_NOINTERFACE;
}

STDMETHODIMP_(BOOL) CDB_BackupImpl::OnMessage(DWORD dwCode, LPARAM lParamW, LPARAM lParamL)
{
	BOOL bManualSave = FALSE;
	LPCTSTR lpDBAbsPath=NULL;
	TCHAR tcPath1[MAX_PATH] = {0};

	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	HWND h = DBBckp_S->g_pAPI->GetMainWindowHandle();

	switch(dwCode)
	{
	case KPM_DIRECT_EXEC:
		My_ProcessDirectCall(lParamW, lParamL);
		break;

	case KPM_DELAYED_INIT:
		InitConfig();
		break;
	case KPM_DIRECT_CONFIG:
		// we need to maintain the %DBDIR% local env variable
		lpDBAbsPath = DBBckp_S->g_pAPI->GetAbsoluteDatabasePath();
		_tcsncpy_s(tcPath1, MAX_PATH, lpDBAbsPath, (_tcsrchr ( lpDBAbsPath, _T('\\') ) - lpDBAbsPath) >> DBBckp_S->shiftBy);
		_tputenv_s(_T("DBDIR"), tcPath1 );
		ShowOptionsDialog(h);
		break;
	
	case KPM_PLUGIN_INFO:
		MessageBox(h, _TRN(SM_ABOUT), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONINFORMATION);
		break;

	case KPM_FILE_CLOSE_POST:
		g_menuItems[1].dwFlags = KPMIF_DISABLED;
		break;

	case KPM_OPENDB_COMMITTED:
		g_menuItems[1].dwFlags = KPMIF_ENABLED;
		lpDBAbsPath = DBBckp_S->g_pAPI->GetAbsoluteDatabasePath();
		if (g_menuItems[4].dwState == KPMIS_CHECKED) 
			BackupDB (bManualSave, lpDBAbsPath, DBBckp_S->tcOpenPrfx);
		break;

	case KPM_SAVEDB_NOW:
		bManualSave = TRUE;

	case KPM_SAVEDB_AS_POST:
		// just a hook to catch this event as well. The processing will continue as for
		// normal save operations
	case KPM_SAVEDB_POST:
		// Backup routine here
		if ((g_menuItems[3].dwState == KPMIS_UNCHECKED) && (!bManualSave))
			break;
		else
			BackupDB (bManualSave, (LPCTSTR) lParamW, DBBckp_S->tcSavePrfx);
		break;

	case KPM_INIT_MENU_POPUP:
		// TODO: Fill this placeholder
		break;

	default:
		break;
	}
	return TRUE;
}

STDMETHODIMP_(LPCTSTR) CDB_BackupImpl::GetProperty(LPCTSTR lpName)
{
	if(lpName == NULL) return NULL;

	if(_tcscmp(lpName, KPPS_COMMANDLINEARGPREFIX) == 0)
		return CMDLINE_PREFIX;

	return NULL;
}

STDMETHODIMP CDB_BackupImpl::SetProperty(LPCTSTR lpName, LPCTSTR lpValue)
{
	// TODO: Implement method
	UNREFERENCED_PARAMETER(lpName);
	UNREFERENCED_PARAMETER(lpValue);
	return E_FAIL;
}

STDMETHODIMP_(DWORD) CDB_BackupImpl::GetMenuItemCount()
{
   return DB_BACKUP_MENU_ITEM_COUNT;
}

STDMETHODIMP_(KP_MENU_ITEM*) CDB_BackupImpl::GetMenuItems()
{
   return &g_menuItems[0];
}

// -----------------------------------------------------------------------------
// DB_Backup specific private functions
// -----------------------------------------------------------------------------
void My_ProcessDirectCall(LPARAM lParamW, LPARAM lParamL)
{
	TCHAR tcMessageHolder[250];
	UNREFERENCED_PARAMETER(lParamL);
	
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	HWND h = DBBckp_S->g_pAPI->GetMainWindowHandle();

	IKpConfig* pCfg = NULL;
	
	
	if(lParamW == (LPARAM)g_menuItems[1].dwCommandID)
		myClass->OnMessage(KPM_SAVEDB_NOW, (LPARAM) DBBckp_S->g_pAPI->GetAbsoluteDatabasePath(), 0);
	else if (lParamW == (LPARAM)g_menuItems[3].dwCommandID) {
		g_menuItems[3].dwState = g_menuItems[3].dwState ^ KPMIS_CHECKED;
		// Additionaly write back to the ini File
		if (!DBBckp_S->bDestCmdParam) {
			if(DBBckp_S->g_pAPI->CreateInstance(CLSID_KpConfig, IID_IKpConfig, (void**)&pCfg) != S_OK) assert(FALSE);
			if (pCfg->SetBool(_T("DB_Backup.BackupOnSave"), (BOOL) g_menuItems[3].dwState) == FALSE ) {
				_sntprintf_s(tcMessageHolder, 250, _TRN(SM_SETERR), _T("DB_Backup.BackupOnSave"));
				MessageBox(h, tcMessageHolder, DBBckp_S->tcPluginTitle, MB_OK | MB_ICONERROR);
			}
			// TODO: Check, why isn't the line in the ini file deleted.
			// Remove the following line after 1.6.0.0+
			pCfg->Set(_T("DB_Backup.BackupsEnabled"), NULL); // removing the old definition, if there's such
			VERIFY(pCfg->Release() == 0);
			if (g_menuItems[3].dwState)
				DBBckp_S->g_pAPI->SetStatusBarText(_TRN(SM_AUTBCKON));
			else
				DBBckp_S->g_pAPI->SetStatusBarText(_TRN(SM_AUTBCKOFF));
		}
	}
	else if(lParamW == (LPARAM)g_menuItems[4].dwCommandID) {
		g_menuItems[4].dwState = g_menuItems[4].dwState ^ KPMIS_CHECKED;
		// Additionaly write back to the ini File
		if (!DBBckp_S->bOpenPrfxCmdParam) {
			if(DBBckp_S->g_pAPI->CreateInstance(CLSID_KpConfig, IID_IKpConfig, (void**)&pCfg) != S_OK) assert(FALSE);
			if (pCfg->SetBool(_T("DB_Backup.BackupOnOpen"), (BOOL) g_menuItems[4].dwState) == FALSE ) {
				_sntprintf_s(tcMessageHolder, 250, _TRN(SM_SETERR), _T("DB_Backup.BackupOnOpen"));
				MessageBox(h, tcMessageHolder, DBBckp_S->tcPluginTitle, MB_OK | MB_ICONERROR);
			}
			VERIFY(pCfg->Release() == 0);
		}
	}
	else if(lParamW == (LPARAM)g_menuItems[5].dwCommandID) {
		// Open Dialog ... or somethin'
		myClass->OnMessage(KPM_DIRECT_CONFIG,0, 0);
	}
	else if(lParamW == (LPARAM)g_menuItems[7].dwCommandID)
		myClass->OnMessage(KPM_PLUGIN_INFO, 0, 0);
	
}


/* *********************************************************** */
// Main Backup function - do the backups itself               //
/* *********************************************************** */
void BackupDB(BOOL bManualSave, LPCTSTR lptDBPath, LPCTSTR lptBackupPrefix) {
	TCHAR tcBuffer[SI_REGSIZE] = {0};
	TCHAR tcErrMsg[500] = {0};
	BOOL bBackupError = FALSE;
	int indx=0;
	IKpConfig* p = NULL;
	IKpUtilities *pUtils = NULL;
	std_string strPath1, strPath2;

	if ((lptDBPath== NULL) || (lptBackupPrefix==NULL)) return;
	if(lptDBPath[0]== _T('\0') || lptBackupPrefix == _T('\0')) return;

	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	HWND h = DBBckp_S->g_pAPI->GetMainWindowHandle();

	HINSTANCE hInst=(HINSTANCE) 1;

	DBBckp_S->g_pAPI->SetStatusBarText(_TRN(SM_BCKSTART));
	// strip the file name from the path
	_tcsncpy_s(tcBuffer, SI_REGSIZE, lptDBPath, (_tcsrchr ( lptDBPath, _T('\\') ) - lptDBPath) >> DBBckp_S->shiftBy);
	// keeping DBDIR env variable updated all the time
	_tputenv_s(_T("DBDIR"), tcBuffer);

	if(DBBckp_S->g_pAPI->CreateInstance(CLSID_KpConfig_ReadOnly, IID_IKpConfig, (void**)&p) != S_OK) assert(FALSE);
	if(DBBckp_S->g_pAPI->QueryInstance (SCLSID_KpUtilities, IID_IKpUtilities, (void**)&pUtils) != S_OK) assert(FALSE);
	// Get first destination (either from the cmdline or the ini file) and expand the env. variables
	strPath1 = ExpandEnvironmentEx(GetNextDestination (p, TRUE));
	while( strPath1.length() != 0 ) {
		// Check if the path is ended with a slash 
		// and concatenate only the FILENAME part of lpDBAbsPath (contains the FULL path of the DB)
		if(strPath1.at(strPath1.length()-1)==_T('\\'))
			_sntprintf_s (tcBuffer, SI_REGSIZE, _TRUNCATE, _T("%s%s%s-"), strPath1.c_str(), lptBackupPrefix, _tcsrchr(lptDBPath, _T('\\') ) +1 );
		else
			_sntprintf_s (tcBuffer, SI_REGSIZE, _TRUNCATE, _T("%s\\%s%s-"), strPath1.c_str(), lptBackupPrefix, _tcsrchr(lptDBPath, _T('\\') ) +1 );
		#ifdef _MYDEBUG
		fprintf(DBBckp_S->fout,"BackupDB 1:Backslash added\ntcNewFileName=%s\n",tcPath);
		#endif
		DoHouseKeeping(DBBckp_S->iBackupCnt, tcBuffer);

		// Prepare the absolute path of the new backup file with the postfix
		strPath1 = std_string(tcBuffer).append( _T("0"));
		
		// Check whether destination directory exists
		strPath2 = strPath1.substr(0,strPath1.rfind(_T('\\')));
		if (pUtils->ValidatePath(strPath2.c_str(), KPVPF_MUST_EXIST || KPVPF_TYPE_DIRECTORY) == FALSE) {
			_sntprintf_s (tcErrMsg, 500, _TRUNCATE, _TRN(SM_CREDIR), strPath2.c_str());
			if(MessageBox(h, tcErrMsg, DBBckp_S->tcPluginTitle, MB_YESNO | MB_DEFBUTTON1 | MB_ICONQUESTION) == IDYES)
				if(pUtils->CreateDirectoryTree(strPath2.c_str(), 0) != S_OK)
					DBBckp_S->g_pAPI->SetStatusBarText(_TRN(SM_DIRFAIL));
		}
		
		/* *********************************************************** */
		// Backup the DB file
		/* *********************************************************** */
		if ( CopyFile ( lptDBPath, strPath1.c_str(), FALSE) == FALSE ) {
			_sntprintf_s (tcErrMsg, 500, _TRUNCATE, _TRN(SM_BCKCPYFAIL), strPath1.c_str());
			MessageBox(h, tcErrMsg, DBBckp_S->tcPluginTitle, MB_OK | MB_ICONERROR);
			bBackupError = TRUE;
		} else {
			// Copy was successful, let's do the rest
			#ifdef _MYDEBUG
			fprintf(DBBckp_S->fout,"BackupDB 2: CopyOK\n ");
			#endif
			_sntprintf_s (tcErrMsg, 500, _TRUNCATE, _TRN(SM_BCKDESTX), indx);
			DBBckp_S->g_pAPI->SetStatusBarText(tcErrMsg);
			
			// Expand environment variables in the external program and add quotes
			strPath2 = std_string( _T("\"") + ExpandEnvironmentEx(std_string(DBBckp_S->tcStartPrg)) + _T("\""));
			#ifdef _MYDEBUG
			fprintf(DBBckp_S->fout,"BackupDB 3: ExpandEnvironment\ntcNewFileName=%s\n", strPath2.c_str());
			#endif

			// Add quotation marks for the first parameter (the backed up file)
			strPath1.insert(0, _T("\""));strPath1.append(_T("\""));
			#ifdef _MYDEBUG
			fprintf(DBBckp_S->fout,"BackupDB 5: Backslash added2\ntcNewFileName=%s\n",strPath1.c_str());
			#endif
			if ( DBBckp_S->bPrgCmdParam || DBBckp_S->bStartExtPrg) {
				#ifdef _MYDEBUG
				fprintf(DBBckp_S->fout,"BackupDB 6: Stating external app.\n");
				#endif
				DBBckp_S->g_pAPI->SetStatusBarText(_TRN(SM_STEXTPRG));
				hInst = ShellExecute(h, NULL, strPath2.c_str(), strPath1.c_str(), NULL, DBBckp_S->bShowWnd ? SW_SHOW : SW_HIDE);
				#ifdef _MYDEBUG
				fprintf(DBBckp_S->fout,"BackupDB 7: External app started.\n");
				#endif
				if ((int) hInst <= 32) {
					bBackupError = TRUE;
					switch((int) hInst) {
					case 0:
						MessageBox(h, _TRN(SM_NORES), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONERROR);
						break;
					case ERROR_FILE_NOT_FOUND:
						MessageBox(h, _TRN(SM_PRGNOTFND), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONERROR);
						break;
					case SE_ERR_ACCESSDENIED:
						MessageBox(h, _TRN(SM_ACCDENIED), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONERROR);
						break;
					}
				}
			}
		}
		indx++;
		// Get next destination (either from the cmdline or the ini file)
		strPath1 = ExpandEnvironmentEx(GetNextDestination (p, FALSE));
		#ifdef _MYDEBUG
		if(tcPath1!=NULL)
			fprintf(DBBckp_S->fout,"BackupDB 8: New destination is:\ntcPath1=%s\n",strPath1.c_str());
		else
			fprintf(DBBckp_S->fout,"BackupDB 9: New destination is NULL\n");
		#endif
	}

	if (p != NULL)
		VERIFY (p->Release()==0);
	if (pUtils != NULL)
		VERIFY (pUtils->Release());

	if  (indx==0) {
		if (!DBBckp_S->bDestCmdParam) {
			if(DBBckp_S->g_pAPI->CreateInstance(CLSID_KpConfig, IID_IKpConfig, (void**)&p) != S_OK) assert(FALSE);
			p->Set(_T("DB_Backup.Destination.1"), _T("") );
			if (p != NULL) VERIFY (p->Release()==0);
			MessageBox(h, _TRN (SM_BCKCANCEL), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONWARNING);
		} 
		else
			MessageBox(h, _TRN(SM_BCKCANCEL), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONWARNING);
	}

	if (bBackupError) {
		DBBckp_S->g_pAPI->SetStatusBarText(_TRN(SM_BCKERR));
		if (bManualSave)
			MessageBox(h, _TRN(SM_BCKERR), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONWARNING);
	} else {
		DBBckp_S->g_pAPI->SetStatusBarText(_TRN(SM_BCKSUC));
		if (bManualSave)
			MessageBox(h, _TRN(SM_BCKSUC), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONWARNING);
	}
	
	bBackupError = FALSE;
	bManualSave = FALSE;
}

/* *********************************************************** */
// Do the housekeeping - rotate the backups                    //
/* *********************************************************** */	
void DoHouseKeeping(int iNumOfBackups, const TCHAR *tcNewFileName) {
TCHAR tcBckNum[4]={0};
std_string strSrc, strDest;

	for (int bckNum = iNumOfBackups - 2; bckNum >= 0; bckNum--) {
		_ultot_s(bckNum, tcBckNum, 4, 10);
		strSrc = std_string(tcNewFileName).append(tcBckNum);
		_ultot_s(bckNum+1, tcBckNum, 4, 10);
		strDest = std_string(tcNewFileName).append(tcBckNum);
		DeleteFile(strDest.c_str());
		MoveFile(strSrc.c_str(), strDest.c_str());
	}
}

std_string GetNextDestination(IKpConfig *pCfg, BOOL bReset) {
static int counter;
TCHAR tcKeyName[INIKEYLEN] = {0};
TCHAR dest[SI_REGSIZE] ={0};
std_string strBuf;


	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	if ( counter > 99 )
		return strBuf;
	if (bReset == TRUE ) counter = 0;

	if ( DBBckp_S->bDestCmdParam ) {
		// return the data from the command line
		#ifdef _MYDEBUG
		fprintf(DBBckp_S->fout,"GetNextDestination - 1 : Retrieving destination from the CmdLine.\n");
		#endif
		strBuf = GetDestFromCmd("dest:", counter);
	}
	else {	
		#ifdef _MYDEBUG
		fprintf(DBBckp_S->fout,"GetNextDestination - 1 : Retrieving destination from the INI file.\n");
		#endif
		_sntprintf_s (tcKeyName, INIKEYLEN, _TRUNCATE, _T("DB_Backup.Destination.%d"), counter+1);
		pCfg->Get(tcKeyName, dest);
		strBuf = (dest[0] == _T('\0') ? std_string() : std_string(dest));
	}

	if (!strBuf.empty()) {
		counter++;
		#ifdef _MYDEBUG
		fprintf(DBBckp_S->fout,"GetNextDestination - 2 : Destination is:\n%s\n", strBuf.c_str());
		#endif
	}
	return strBuf;
}


void InitConfig(void) {
TCHAR tcBuffer[SI_REGSIZE] = {0};
int val = 0;
IKpConfig* p = NULL;
	
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();

	// Initialize the variables
	//GetModuleFileName(DBBckp_S->hInstDLL, DBBckp_S->tcAppBasePath, MAX_PATH);
	ParseCommandLine();

	_tputenv_s(_T("KEEPASSDIR"), DBBckp_S->tcAppBasePath);


	if(DBBckp_S->g_pAPI->CreateInstance(CLSID_KpConfig_ReadOnly, IID_IKpConfig, (void**)&p) != S_OK) assert(FALSE);

	if (DBBckp_S->iBackupCnt == 0) {
		p->Get(_T("DB_Backup.KeepXBackups"), tcBuffer);
		if(_tcslen(tcBuffer) > 0 )
			DBBckp_S->iBackupCnt = atoi(tcBuffer);
		else
			DBBckp_S->iBackupCnt = 1;
	}

	if (!DBBckp_S->bOpenPrfxCmdParam) {
		p->Get(_T("DB_Backup.PrefixOnOpen"), tcBuffer);
		if(tcBuffer[0] == _T('\0'))
			_tcsncpy_s(DBBckp_S->tcOpenPrfx, MAX_PATH, DEF_PREFIX_ON_OPEN, _TRUNCATE);
		else
			_tcsncpy_s(DBBckp_S->tcOpenPrfx, MAX_PATH, tcBuffer, _TRUNCATE);
	}

	if (!DBBckp_S->bSavePrfxCmdParam) {
		p->Get(_T("DB_Backup.PrefixOnSave"), tcBuffer);
		if(tcBuffer[0] == _T('\0'))
			_tcsncpy_s(DBBckp_S->tcSavePrfx, MAX_PATH, DEF_PREFIX_ON_SAVE, _TRUNCATE);
		else
			_tcsncpy_s(DBBckp_S->tcSavePrfx, MAX_PATH, tcBuffer, _TRUNCATE);
	}

	if (!DBBckp_S->bPrgCmdParam) {
		p->Get(_T("DB_Backup.ExtPrgPath"), tcBuffer);
		_tcsncpy_s(DBBckp_S->tcStartPrg, MAX_PATH, tcBuffer, _TRUNCATE);
		DBBckp_S->bStartExtPrg = p->GetBool(_T("DB_Backup.StartExtPrg"), FALSE);
	}
	else
		DBBckp_S->bStartExtPrg = TRUE;

	if (!DBBckp_S->bWndCmdParam) {
		val = p->GetBool(_T("DB_Backup.ShowWindow"), FALSE);
		DBBckp_S->bShowWnd = (val > 0 ? TRUE : FALSE);
	}

	if (!DBBckp_S->bSavePrfxCmdParam)
		g_menuItems[3].dwState = p->GetBool(_T("DB_Backup.BackupOnSave"), TRUE);
		if (g_menuItems[3].dwState != KPMIS_CHECKED)
			// Leaving here for backward compatibility
			g_menuItems[3].dwState = p->GetBool(_T("DB_Backup.BackupsEnabled"), TRUE);
	else
		g_menuItems[3].dwState = KPMIS_CHECKED;

	if (!DBBckp_S->bOpenPrfxCmdParam)
		g_menuItems[4].dwState = p->GetBool(_T("DB_Backup.BackupOnOpen"), FALSE);
	else
		g_menuItems[4].dwState = KPMIS_CHECKED;

	VERIFY(p->Release() == 0);
}