/*
  Copyright (c) 2009, Matyas Bene <notabenem@hotmail.com>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer. 
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Neither the name of ReichlSoft nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/


#include "Options.h"
#include "DB_Backup.h"
#include "resource.h"
#include "utils.h"

#include <tchar.h>
#include <shlobj.h>
#include <stdio.h>
#include <assert.h>
#ifdef _DEBUG
#define VERIFY(f)          assert(f)
#else
#define VERIFY(f)          ((void)(f))
#endif

int iDirsConfigured = 0;


BOOL MainDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
LRESULT lResult;
int indx=0;
IKpConfig *pCfg;
std_string strBuf;
	
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	switch (uMsg)
	{
	case WM_INITDIALOG:
		// Localization
		// Enum all the controls on the dialogue, and set their captions
		LocalizeDialog(hDlg);

		// Fill the listbox with existing values from the INI file
		indx=1;
		if(DBBckp_S->g_pAPI->CreateInstance(CLSID_KpConfig_ReadOnly, IID_IKpConfig, (void**)&pCfg) != S_OK) assert(FALSE);
		strBuf = GetNextDestination (pCfg, TRUE);

		while( (!strBuf.empty()) && (indx < 100) ) {
			lResult = SendDlgItemMessage(hDlg, IDC_DESTS, LB_ADDSTRING, 0, (LPARAM) strBuf.c_str());
			strBuf = GetNextDestination (pCfg, FALSE);
			indx++;
		}
		VERIFY(pCfg->Release() == 0);
		iDirsConfigured = indx;
		
		// Number of backups to keep
		SetDlgItemInt(hDlg, IDC_BNUM, DBBckp_S->iBackupCnt, FALSE);
		
		// Prefixes
		SetDlgItemText(hDlg, IDC_PREFO, DBBckp_S->tcOpenPrfx);
		SetDlgItemText(hDlg, IDC_PREFS, DBBckp_S->tcSavePrfx);

		//External program to start
		SetDlgItemText(hDlg, IDC_STARTPRG, DBBckp_S->tcStartPrg);

		EnableWindow(GetDlgItem(hDlg, IDC_ADD), !DBBckp_S->bDestCmdParam);
		EnableWindow(GetDlgItem(hDlg, IDC_REMOVE), !DBBckp_S->bDestCmdParam);
		EnableWindow(GetDlgItem(hDlg, IDC_BNUM), !DBBckp_S->bCountCmdParam);
		EnableWindow(GetDlgItem(hDlg, IDC_SHOWWND), !DBBckp_S->bWndCmdParam);

		if ( DBBckp_S->bPrgCmdParam ) {
			CheckDlgButton(hDlg, IDC_ENABLEPRG, BST_CHECKED);
			EnableWindow(GetDlgItem(hDlg, IDC_ENABLEPRG), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_STARTPRG), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_BROWSE2), FALSE);
		} 
		else {
			if (DBBckp_S->bStartExtPrg == TRUE) {
				CheckDlgButton(hDlg, IDC_ENABLEPRG, BST_CHECKED);
				EnableWindow(GetDlgItem(hDlg, IDC_STARTPRG), TRUE);
				EnableWindow(GetDlgItem(hDlg, IDC_BROWSE2), TRUE);
				if (!DBBckp_S->bWndCmdParam)
					EnableWindow(GetDlgItem(hDlg, IDC_SHOWWND), TRUE);
			}
			else {
				CheckDlgButton(hDlg, IDC_ENABLEPRG, BST_UNCHECKED);
				EnableWindow(GetDlgItem(hDlg, IDC_STARTPRG), FALSE);
				EnableWindow(GetDlgItem(hDlg, IDC_BROWSE2), FALSE);
				if (!DBBckp_S->bWndCmdParam)
					EnableWindow(GetDlgItem(hDlg, IDC_SHOWWND), FALSE);
			}
		}
		
		if (DBBckp_S->bShowWnd)
			CheckDlgButton(hDlg, IDC_SHOWWND, BST_CHECKED);
		else
			CheckDlgButton(hDlg, IDC_SHOWWND, BST_UNCHECKED);
		
		if (DBBckp_S->bDestCmdParam || DBBckp_S->bPrgCmdParam || DBBckp_S->bCountCmdParam || DBBckp_S->bWndCmdParam)
			MessageBox(hDlg, _TRN(SM_PARAMLCKD), DBBckp_S->tcPluginTitle, MB_OK | MB_ICONINFORMATION);
		return TRUE;
	case WM_NOTIFY:
        //SetDlgState(hDlg);
        //return TRUE;
		break;
	case WM_COMMAND:
		return MainDialog_OnCommand(hDlg, LOWORD(wParam), HIWORD(wParam), (HWND)lParam);
	case WM_CLOSE:
		EndDialog(hDlg, 0);
		return TRUE;
	}
	return FALSE;
}

int ShowOptionsDialog(HWND hDlg)
{
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	DialogBoxParam(DBBckp_S->hInstDLL, MAKEINTRESOURCE(IDD_DLG1), 
		hDlg, (DLGPROC)MainDialogProc, 0);
	return 0;
}

BOOL MainDialog_OnCommand(HWND hDlg, WORD wCommand, WORD wNotify, HWND hControl)
{
TCHAR tcBuffer[SI_REGSIZE]={0};
TCHAR tcExpandedPath[MAX_PATH]={0};
TCHAR tcKeyName[INIKEYLEN]={0};
LRESULT lResult = 0;
BROWSEINFO bi;
OPENFILENAME ofn;
LPCITEMIDLIST pidl;
BOOL Translated;
LPCTSTR tcFilter=_T("All Files(*.*)\0*.*\0\0");
IKpConfig *pCfg;

	UNREFERENCED_PARAMETER(hControl);
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	switch (wCommand)
	{
	case IDC_DEST:
		//  (wNotify == LBN_KILLFOCUS) || (wNotify == CBN_KILLFOCUS) ||
		//	(wNotify == NM_KILLFOCUS)  || (wNotify == WM_KILLFOCUS))
		if (wNotify == EN_KILLFOCUS) {
			GetDlgItemText(hDlg, IDC_DEST, tcBuffer, sizeof(tcBuffer) >> DBBckp_S->shiftBy );
			LookForRelativePath(hDlg, tcBuffer, IDC_DEST);
		}
		break;
	case IDC_STARTPRG:
		if (wNotify == EN_KILLFOCUS) {
			GetDlgItemText(hDlg, IDC_STARTPRG, tcBuffer, sizeof(tcBuffer) >> DBBckp_S->shiftBy );
			LookForRelativePath(hDlg, tcBuffer, IDC_STARTPRG);
		}
		break;
	case IDC_OK:
		if(DBBckp_S->g_pAPI->CreateInstance(CLSID_KpConfig, IID_IKpConfig, (void**)&pCfg) != S_OK) assert(FALSE);
		if (!DBBckp_S->bDestCmdParam) {
			lResult = SendDlgItemMessage(hDlg, IDC_DESTS, LB_GETCOUNT, 0, 0);
			int indx;
			for  (indx=0; indx < lResult; indx++ ) {
				//if (SendDlgItemMessage(hDlg, IDC_DESTS, LB_GETTEXTLEN, indx, NULL )>SI_REGSIZE) continue;
				SendDlgItemMessage(hDlg, IDC_DESTS, LB_GETTEXT, indx, (LPARAM) tcBuffer);
				_sntprintf_s (tcKeyName, INIKEYLEN, _TRUNCATE, _T("DB_Backup.Destination.%d"), indx + 1);
				VERIFY(pCfg->Set(tcKeyName, tcBuffer));
			}
			for  (indx=lResult; indx < iDirsConfigured ; indx++ ) {
				_sntprintf_s (tcKeyName, INIKEYLEN, _TRUNCATE, _T("DB_Backup.Destination.%d"), indx + 1);
				VERIFY(pCfg->Set(tcKeyName, NULL));
			}
		}
		if (!DBBckp_S->bCountCmdParam) {
			lResult = GetDlgItemInt(hDlg, IDC_BNUM, &Translated, FALSE);
			DBBckp_S->iBackupCnt = ((Translated) ? lResult : 1);
			_sntprintf_s (tcKeyName, INIKEYLEN, _TRUNCATE, "%d", DBBckp_S->iBackupCnt);
			VERIFY(pCfg->Set(_T("DB_Backup.KeepXBackups"), tcKeyName));
		}

		// Prefixes
		if (!DBBckp_S->bOpenPrfxCmdParam) {
			GetDlgItemText(hDlg, IDC_PREFO, DBBckp_S->tcOpenPrfx, MAX_PATH );
			VERIFY(pCfg->Set(_T("DB_Backup.PrefixOnOpen"), DBBckp_S->tcOpenPrfx));
		}

		if (!DBBckp_S->bSavePrfxCmdParam) {
			GetDlgItemText(hDlg, IDC_PREFS, DBBckp_S->tcSavePrfx, MAX_PATH );
			VERIFY(pCfg->Set(_T("DB_Backup.PrefixOnSave"), DBBckp_S->tcSavePrfx));
		}

		if (!DBBckp_S->bPrgCmdParam) {
			DBBckp_S->bStartExtPrg = ((IsDlgButtonChecked(hDlg, IDC_ENABLEPRG) == BST_CHECKED) ? TRUE : FALSE);
			pCfg->SetBool(_T("DB_Backup.StartExtPrg"), DBBckp_S->bStartExtPrg);
			GetDlgItemText(hDlg, IDC_STARTPRG, DBBckp_S->tcStartPrg, MAX_PATH );
			VERIFY(pCfg->Set(_T("DB_Backup.ExtPrgPath"), DBBckp_S->tcStartPrg));
		}

		if (!DBBckp_S->bWndCmdParam) {
			DBBckp_S->bShowWnd = ((IsDlgButtonChecked(hDlg, IDC_SHOWWND) == BST_CHECKED) ? TRUE : FALSE);
			VERIFY(pCfg->SetBool(_T("DB_Backup.ShowWindow"), DBBckp_S->bShowWnd));
		}
		VERIFY(pCfg->Release() == 0);
		EndDialog(hDlg, wCommand);
		break;
		
	case IDC_BROWSE:
		//SHBrowseForFolder
		GetDlgItemText(hDlg, IDC_DEST, tcBuffer, sizeof(tcBuffer) >> DBBckp_S->shiftBy );
		_tcsncpy_s(tcExpandedPath, sizeof(tcExpandedPath) >> DBBckp_S->shiftBy, 
			ExpandEnvironmentEx(std_string(tcBuffer)).c_str(), _TRUNCATE);
		bi.hwndOwner = hDlg;
		bi.pidlRoot = 0;
		bi.pszDisplayName = tcExpandedPath;
		bi.lpszTitle = _TRN (SD_BRWSFLD);
		bi.ulFlags = 0;
		bi.lpfn = 0;
		bi.lParam = 0;
		bi.iImage = 0;

		if((pidl = SHBrowseForFolder(&bi)) != 0) {
			SHGetPathFromIDList(pidl, tcBuffer);
			LookForRelativePath(hDlg, tcBuffer, IDC_DEST );
		}
		break;
		
	case IDC_BROWSE2:
		GetDlgItemText(hDlg, IDC_STARTPRG, tcBuffer, sizeof(tcBuffer) >> DBBckp_S->shiftBy );
		_tcsncpy_s(tcExpandedPath, sizeof(tcExpandedPath) >> DBBckp_S->shiftBy, 
			ExpandEnvironmentEx(std_string(tcBuffer)).c_str(), _TRUNCATE);
		ofn.lStructSize = sizeof (OPENFILENAME); //OPENFILENAME_SIZE_VERSION_400
		ofn.hwndOwner = hDlg;
		ofn.lpstrFilter = tcFilter;
		ofn.nFilterIndex = 0;
		ofn.lpstrCustomFilter = NULL;
		ofn.lpstrFile = NULL;
		ofn.lpstrFile = tcExpandedPath;
		ofn.nMaxFile = sizeof(tcExpandedPath) >> DBBckp_S->shiftBy;
		ofn.lpstrFileTitle = NULL;
		ofn.lpstrInitialDir = NULL;
		ofn.lpstrTitle = NULL;
		ofn.lpstrDefExt = NULL;
		ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
		if(!GetOpenFileName(&ofn)) break;
		LookForRelativePath(hDlg, tcExpandedPath, IDC_STARTPRG );
		break;
		
	case IDC_DESTS:
		if (wNotify ==  LBN_SELCHANGE) {
			lResult = SendDlgItemMessage(hDlg, IDC_DESTS, LB_GETCURSEL, 0, 0);
			if ( lResult != LB_ERR ) {
				SendDlgItemMessage(hDlg, IDC_DESTS, LB_GETTEXT, lResult, (LPARAM) tcBuffer);
				SetDlgItemText(hDlg, IDC_DEST, tcBuffer);
			}
		}
		break;
		
	case IDC_ADD:
		lResult = SendDlgItemMessage(hDlg, IDC_DESTS, LB_GETCOUNT, 0, 0);

		if (lResult == LB_ERR ) break;
		if (lResult == 99) {
			MessageBox(hDlg, _TRN(SM_MAXBACK), DBBckp_S->tcPluginTitle, MB_OK);
		}
		else
			GetDlgItemText(hDlg, IDC_DEST, tcBuffer, sizeof(tcBuffer) >> DBBckp_S->shiftBy );
			if (_tcslen(tcBuffer) == 0 ) break;
			lResult = SendDlgItemMessage(hDlg, IDC_DESTS, LB_ADDSTRING, 0, (LPARAM) tcBuffer);
		break;
		
	case IDC_REMOVE:
		lResult = SendDlgItemMessage(hDlg, IDC_DESTS, LB_GETCURSEL, 0, 0);
		if (lResult == LB_ERR) {
			MessageBox(hDlg, _TRN(SM_NOSEL), DBBckp_S->tcPluginTitle, MB_OK);
		}
		else
			SendDlgItemMessage(hDlg, IDC_DESTS, LB_DELETESTRING, lResult, 0);
		break;
		
	case IDC_ENABLEPRG:
		if (IsDlgButtonChecked(hDlg, IDC_ENABLEPRG) == BST_CHECKED) {
			EnableWindow(GetDlgItem(hDlg, IDC_STARTPRG), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_BROWSE2), TRUE);
			if (!DBBckp_S->bWndCmdParam)
				EnableWindow(GetDlgItem(hDlg, IDC_SHOWWND), TRUE);
		}
		else {
			EnableWindow(GetDlgItem(hDlg, IDC_STARTPRG), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_BROWSE2), FALSE);
			if (!DBBckp_S->bWndCmdParam)
				EnableWindow(GetDlgItem(hDlg, IDC_SHOWWND), FALSE);
		}
		break;
		
	case IDC_CANCEL:
		EndDialog(hDlg, wCommand);
		break;
	}
	return TRUE;
}


void LookForRelativePath(HWND hDlg, LPCTSTR tcPath, LONG destControlID) {
TCHAR tcbuffer[MAX_PATH]={0};
TCHAR tcDBPath[MAX_PATH]={0};
TCHAR *tcPos;

	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	if (tcPath[0] == _T('\0')) {
		SetDlgItemText(hDlg, destControlID, "");
		return;
	}

	if (_tcsstr( tcPath, _T(":\\")) != NULL ) {
		// This is an absolute path, let's see, if it can be converted to a relative one
		tcPos = _tcsstr((TCHAR *) tcPath, DBBckp_S->tcAppBasePath);
		if (tcPos != NULL) {
			// This is a path under the KeePass install directory
			// we should ask the user, whether he wants it to be relative
			if (MessageBox(hDlg, _TRN(SM_RELPATH), DBBckp_S->tcPluginTitle, 
					MB_YESNO | MB_DEFBUTTON1 | MB_ICONQUESTION) == IDYES) {
				if ( _tcslen(tcPath) > _tcslen(DBBckp_S->tcAppBasePath) ) {
					_sntprintf_s (tcbuffer, MAX_PATH, MAX_PATH-1, _T("%%KEEPASSDIR%%\\%s"), tcPath + ((_tcslen(DBBckp_S->tcAppBasePath)+1) << DBBckp_S->shiftBy));
					SetDlgItemText(hDlg, destControlID, tcbuffer);
					return;
				}
				else {
					SetDlgItemText(hDlg, destControlID, _T("%KEEPASSDIR%"));
					return;
				}
			} 
		} else {
			tcPos = (TCHAR *) DBBckp_S->g_pAPI->GetAbsoluteDatabasePath();
			if (tcPos[0]!= _T('\0') ) {
				// stripping the DB file name (*.kdb), the remaining part is the
				// absolute directory path, where the file resides
				_tcsncpy_s(tcDBPath, MAX_PATH, tcPos, (_tcsrchr ( tcPos, '\\') - tcPos) >> DBBckp_S->shiftBy);
				tcPos = _tcsstr((TCHAR *) tcPath, tcDBPath);
				if (tcPos != NULL) {
					// This is a path relative to the DB
					if (MessageBox(hDlg, _TRN(SM_RELPATH), DBBckp_S->tcPluginTitle, 
							MB_YESNO | MB_DEFBUTTON1 | MB_ICONQUESTION) == IDYES) {
						if ( _tcslen(tcPath) > _tcslen(tcDBPath) ) {
							_sntprintf_s (tcbuffer, MAX_PATH, MAX_PATH-1, _T("%%DBDIR%%\\%s"), tcPath + ((_tcslen(tcDBPath)+1) << DBBckp_S->shiftBy));
							SetDlgItemText(hDlg, destControlID, tcbuffer);
							return;
						} else {
							SetDlgItemText(hDlg, destControlID, _T("%DBDIR%"));
							return;
						}
					}
				}
			}
		}
	}
	SetDlgItemText(hDlg, destControlID, tcPath);
	return;
}

void LocalizeDialog(HWND hDlg) {
/*
	if((strstr(_szClassName,"EDIT")==NULL)&&(strstr(_szClassName,"BOX")==NULL))
{
  SetWindowText(hHandle,szTitle);
}
else
{
  HWND hWndParent=GetParent(hHandle);
  SendDlgItemMessage(hWndParent,dwID,WM_SETTEXT,0,(LPARAM)szTitle);
}
*/
	SetWindowText(hDlg, _TRN(SD_TITLE));
	
    SendDlgItemMessage(hDlg, IDC_STC1, WM_SETTEXT,0,(LPARAM) _TRN(SD_DESTINATION));
	SendDlgItemMessage(hDlg, IDC_STC2, WM_SETTEXT,0,(LPARAM) _TRN(SD_LISTDESC));
    SendDlgItemMessage(hDlg, IDC_ADD, WM_SETTEXT,0,(LPARAM) _TRN(SD_ADD));
	SendDlgItemMessage(hDlg, IDC_REMOVE, WM_SETTEXT,0,(LPARAM) _TRN(SD_REMOVE));
	SendDlgItemMessage(hDlg, IDC_STC3, WM_SETTEXT,0,(LPARAM) _TRN(SD_NUMBACK));
	SendDlgItemMessage(hDlg, IDC_STC4, WM_SETTEXT,0,(LPARAM) _TRN(SD_OPENPRFX));
	SendDlgItemMessage(hDlg, IDC_STC5, WM_SETTEXT,0,(LPARAM) _TRN(SD_SAVEPRFX));
	SendDlgItemMessage(hDlg, IDC_ENABLEPRG, WM_SETTEXT,0,(LPARAM) _TRN(SD_STARTPRG));
	SendDlgItemMessage(hDlg, IDC_SHOWWND, WM_SETTEXT,0,(LPARAM) _TRN(SD_SHOWWND));
	SendDlgItemMessage(hDlg, IDC_OK, WM_SETTEXT,0,(LPARAM) _TRN(SD_OK));
	SendDlgItemMessage(hDlg, IDC_CANCEL, WM_SETTEXT,0,(LPARAM) _TRN(SD_CANCEL));
	
	return;
}
