/*
  Copyright (c) 2009, Matyas Bene <notabenem@hotmail.com>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer. 
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Neither the name of ReichlSoft nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/


#include "Utils.h"
#include "DB_Backup.h"
#include "resource.h"
#include <tchar.h>

#include <assert.h>
#ifdef _DEBUG
#define VERIFY(f)          assert(f)
#else
#define VERIFY(f)          ((void)(f))
#endif

TCHAR tcTrnlBuffer[MAXRESNUM-CUSTOMRESNUM+1] [MAXRESSTRLEN + 1]= {0};
TCHAR tcLangFileAbsPath[MAX_PATH];

LPCTSTR InitTranslationEngine (LPCTSTR lpName) {
	TCHAR *tcIndex1;
	TCHAR *tcIndex2;
	TCHAR tcLangName[50];
	IKpCommandLine *pCmdLine;

	// If using separate language dlls
	// hInst = LoadLibrary(_TEXT("res409.dll"));

	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	
	// Get the full path name of the executable
	if(DBBckp_S->g_pAPI->QueryInstance (SCLSID_KpCommandLine, IID_IKpCommandLine, (void**)&pCmdLine) != S_OK) assert(FALSE);
	_tcsncpy_s(DBBckp_S->tcAppBasePath, MAX_PATH, pCmdLine->GetExecutablePath(), _TRUNCATE);	
	tcIndex1 = _tcsrchr ( DBBckp_S->tcAppBasePath, _T('\\') );		// search for the last slash
	tcIndex1[0] = _T('\0');											// remove the file name part
	VERIFY(pCmdLine->Release());


	// Construct the language file name
	_tcsncpy_s(tcLangName, 50, lpName, _TRUNCATE);
	_sntprintf_s(tcLangFileAbsPath, MAX_PATH, _T("%s\\%s_DB_Backup.plng"), DBBckp_S->tcAppBasePath, tcLangName);

	//Check whether the file exists. If not, try to shorten the name
	while ((GetFileAttributes( tcLangFileAbsPath ) == INVALID_FILE_ATTRIBUTES) && (_tcslen(tcLangName)>0 )) {
		tcIndex1 = _tcsrchr ( tcLangName, _T('_') );
		tcIndex1 = (tcIndex1 == NULL? tcLangName: tcIndex1);
		tcIndex2 = _tcsrchr ( tcLangName, _T('-') );
		tcIndex2 = (tcIndex2 == NULL? tcLangName: tcIndex2);
		tcIndex1 = (tcIndex1>tcIndex2?tcIndex1:tcIndex2);
		tcIndex1[0]=_T('\0');
		_sntprintf_s(tcLangFileAbsPath, MAX_PATH, _T("%s\\%s_DB_Backup.plng"), DBBckp_S->tcAppBasePath, tcLangName);
		return tcLangFileAbsPath;
	}
	return NULL;
}

LPTSTR _Translate(UINT UID, LPCTSTR tcStrKeyName) {
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	if ( tcTrnlBuffer[UID-CUSTOMRESNUM][0] == _T('\0') ) {
		// First, check if the corresponding language file is available
		if ( GetPrivateProfileString( _T("DB_Backup"), tcStrKeyName, NULL, tcTrnlBuffer[UID-CUSTOMRESNUM],
			MAXRESSTRLEN+1, tcLangFileAbsPath) == 0 )
				// If the file does not exist, get it from the string table (rc file)
				LoadString(DBBckp_S->hInstDLL, UID, tcTrnlBuffer[UID-CUSTOMRESNUM], MAXRESSTRLEN+1 );
		// Replacing newlines chars with new line
		TCHAR *tcPos=_tcsstr(tcTrnlBuffer[UID-CUSTOMRESNUM], _T("\\n") );
		while (tcPos != NULL) {
				tcPos[0]=_T('\r');
				tcPos[1]=_T('\n');
				tcPos=_tcsstr(tcPos,_T("\\n"));
		}
	}
	return tcTrnlBuffer[UID-CUSTOMRESNUM];
}



////////////////////////////////////////////////////////
// Expand environment Ex routine, using basic_string  //
////////////////////////////////////////////////////////
std_string ExpandEnvironmentEx(std_string s) {
TCHAR *tcTokenStart=NULL;
TCHAR *tcTokenEnd=NULL;
TCHAR *tcHolder=NULL;
TCHAR *tcFragmentStart=NULL;
TCHAR *tcEnvString=NULL;
std_string strExpanded;
size_t len;
size_t requiredSize;
int state=0;
	
	if(s.empty()) return strExpanded;

	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	tcHolder = tcFragmentStart = (TCHAR *)s.c_str();

	while( (tcHolder = _tcschr(tcFragmentStart, _T('%') ))  != NULL )
	{
	  switch (++state) {
	  case 1:
		tcTokenStart = tcHolder;
		len = (tcTokenStart - tcFragmentStart) >> DBBckp_S->shiftBy ;
		if ( len > 0)
		  strExpanded.append(tcFragmentStart, len );
		tcFragmentStart = tcTokenStart + sizeof (TCHAR);
		break;

	  case 2:
		tcTokenEnd = tcHolder;
		len = (tcTokenEnd - tcFragmentStart) >> DBBckp_S->shiftBy ;
		
		// Get Environment variable value in 2 steps
		// First, get the size of the required buffer
		getenv_s( &requiredSize, NULL, 0, std_string(tcFragmentStart).substr(0,len).c_str() );
		if(requiredSize > 0) {
			tcEnvString = (TCHAR *) malloc(requiredSize << DBBckp_S->shiftBy);
			// then, when the buffer is ready, copy the value of the variable into the buffer
			_tgetenv_s(&requiredSize, tcEnvString, requiredSize << DBBckp_S->shiftBy, 
					std_string(tcFragmentStart).substr(0,len).c_str());
		}

		if (tcEnvString != NULL)
			strExpanded.append(tcEnvString);
		else
			// len +2 because we have to copy both '%' signs
			strExpanded.append(std_string(tcTokenStart).substr(0, len+2));

		// and of course, free the variable for future use.
		free ( tcEnvString );
		tcFragmentStart = tcTokenEnd + sizeof (TCHAR);
		state = 0;
		break;
	  }
	}

	if (state==1)
		 tcFragmentStart -= sizeof (TCHAR);
	if (tcFragmentStart[0] != _T('\0'))
		//_tcsncat_s(dest, lBufSizeDest, tcFragment, lBufSizeDest - _tcslen(dest) );
		strExpanded.append(tcFragmentStart);
	return strExpanded;
}



// ******************************************************
// The following code has been contributed by Bill Rubin
// modified by me to suit DB_Backup
// ******************************************************
void displayIgnoredMessage(HWND h, const std_string& argument, const std_string& reason) {
	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	const std_string message(std_string(_T("Command line argument \"")) + argument + _T("\" ") + reason + _T('.'));
    MessageBox(h, message.c_str(), DBBckp_S->tcPluginTitle, MB_ICONERROR);
}


void ParseCommandLine() {

std_string strBuf;
TCHAR tcErrorMsg[200]={0};
LPCTSTR lpOptionValue;
TCHAR tcNumAsStr[55] = {0};
TCHAR *tcNumStart = NULL;
BOOL bNegVal = FALSE;
int counter=0;
IKpCommandLine *pCmdLine;
IKpUtilities *pUtils;

	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();
	HWND h = DBBckp_S->g_pAPI->GetMainWindowHandle();
	if(DBBckp_S->g_pAPI->QueryInstance (SCLSID_KpCommandLine, IID_IKpCommandLine, (void**)&pCmdLine) != S_OK) assert(FALSE);
	if(DBBckp_S->g_pAPI->QueryInstance (SCLSID_KpUtilities, IID_IKpUtilities, (void**)&pUtils) != S_OK) assert(FALSE);

	//_tcsncpy_s(DBBckp_S->tcAppBasePath, MAX_PATH, pCmdLine->GetExecutablePath(), _TRUNCATE);
	//tcNumStart = _tcsrchr ( DBBckp_S->tcAppBasePath, _T('\\') );
	//tcNumStart[0] = _T('\0');

	// For each command line argument 'dest:' 
	while ((lpOptionValue=pCmdLine->GetMultiOption(std_string(CMDLINE_PREFIX).append(_T("dest:")).c_str(), counter++)) != NULL) {
		strBuf = ExpandEnvironmentEx(std_string(lpOptionValue));
		if (!pUtils->ValidatePath(strBuf.c_str(), KPVPF_REGULAR_NAME | KPVPF_DEFAULT))
			displayIgnoredMessage(h, pCmdLine->GetTokenRel(std_string(CMDLINE_PREFIX).append(_T("dest:")).c_str(), counter-1),
				_T("ignored because destination is invalid"));
		else
			DBBckp_S->bDestCmdParam = TRUE;
	}



	// Check for a present command line argument 'cnt:' 
	if ((tcNumStart=(TCHAR *)pCmdLine->GetOption(std_string(CMDLINE_PREFIX).append(_T("cnt:")).c_str())) != NULL) {
		if(tcNumStart[0]==_T('-')) {
			bNegVal = TRUE;
			tcNumStart += sizeof(TCHAR);
		}

		while(tcNumStart[0]==_T('0'))
			tcNumStart += sizeof(TCHAR);
		
		int iCnt = _tstoi(tcNumStart);
		iCnt=(iCnt==0?1:iCnt);
		_itot_s(iCnt, tcNumAsStr, 55, 10);
		if ((_tcscmp(tcNumAsStr, tcNumStart) != 0) || bNegVal ){
			_sntprintf_s (tcErrorMsg, 200, 199, 
				_T("partially ignored because option not entirely of integer type or negative.\n\"%s\" will be used instead"), tcNumAsStr);
			displayIgnoredMessage(h, pCmdLine->GetTokenRel(std_string(CMDLINE_PREFIX).append(_T("cnt:")).c_str(), 0), tcErrorMsg );
		}
		if (iCnt>99) {
			displayIgnoredMessage(h, pCmdLine->GetTokenRel(std_string(CMDLINE_PREFIX).append(_T("cnt:")).c_str(), 0), 
				_T("ignored because it was out of range (>99).\n\"99\" will be used instead") );
			iCnt=99;
		}
		DBBckp_S->iBackupCnt = iCnt;
		DBBckp_S->bCountCmdParam = TRUE;
	}

	// Check if the OpenPrefix is given 'prfxo:'
	if ((lpOptionValue=pCmdLine->GetOption(std_string(CMDLINE_PREFIX).append(_T("prfxo:")).c_str())) != NULL) {
		strBuf = ExpandEnvironmentEx(std_string(lpOptionValue));
		_tcsncpy_s(DBBckp_S->tcOpenPrfx, MAX_PATH, strBuf.c_str(), _TRUNCATE);
		DBBckp_S->bOpenPrfxCmdParam = TRUE;
	}

	// Check if the SavePrefix is given 'prfxs:'
	if ((lpOptionValue=pCmdLine->GetOption(std_string(CMDLINE_PREFIX).append(_T("prfxs:")).c_str())) != NULL) {
		strBuf = ExpandEnvironmentEx(std_string(lpOptionValue));
		_tcsncpy_s(DBBckp_S->tcSavePrfx, MAX_PATH, strBuf.c_str(), _TRUNCATE);
		DBBckp_S->bSavePrfxCmdParam = TRUE;
	}
	
	// Check for a present command line argument 'prog:' 
	if ((lpOptionValue=pCmdLine->GetOption(std_string(CMDLINE_PREFIX).append(_T("prog:")).c_str())) != NULL) {
		strBuf = ExpandEnvironmentEx(std_string(lpOptionValue));
		if (!pUtils->ValidatePath(strBuf.c_str(), KPVPF_REGULAR_NAME | KPVPF_DEFAULT))
			displayIgnoredMessage(h, pCmdLine->GetTokenRel(std_string(CMDLINE_PREFIX).append(_T("prog:")).c_str(), 0), 
				_T("ignored because the program path is invalid"));
		else {
			_tcsncpy_s(DBBckp_S->tcStartPrg, MAX_PATH, strBuf.c_str(), _TRUNCATE);
			DBBckp_S->bPrgCmdParam = TRUE;
		}
	}

	// Check for a present command line argument 'show:' 
	if ((lpOptionValue=pCmdLine->GetOption(std_string(CMDLINE_PREFIX).append(_T("show:")).c_str())) != NULL) {
		int iShow = _tstoi(lpOptionValue);
		DBBckp_S->bShowWnd = (iShow > 0 ? TRUE : FALSE);
		if ((iShow != TRUE) && (iShow != FALSE)){
			_sntprintf_s (tcErrorMsg, 200, 199, 
				_T("partially ignored because option not \"1\" or \"0\"\n\"%d\" will be used instead"), DBBckp_S->bShowWnd);
			displayIgnoredMessage(h, pCmdLine->GetTokenRel(std_string(CMDLINE_PREFIX).append(_T("show:")).c_str(), 0), tcErrorMsg );
		}
		DBBckp_S->bWndCmdParam = TRUE;
	}
	// TODO: Dominik: somehow advice the user of ignored command line options
    // displayIgnoredMessage(h, ignored_argument, _T("ignored because option not recognized"));
	VERIFY(pCmdLine->Release());
	VERIFY(pUtils->Release());
}

std_string GetDestFromCmd(const TCHAR *tcParam, int iParamInstance) {
LPCTSTR lpOptionValue;
std_string strBuf1, strBuf2;
IKpCommandLine *pCmdLine;
IKpUtilities *pUtils;

	DB_Backup_Settings * DBBckp_S=DB_Backup_Settings::GetInstance();

	if(DBBckp_S->g_pAPI->QueryInstance (SCLSID_KpCommandLine, IID_IKpCommandLine, (void**)&pCmdLine) != S_OK) assert(FALSE);
	if(DBBckp_S->g_pAPI->QueryInstance (SCLSID_KpUtilities, IID_IKpUtilities, (void**)&pUtils) != S_OK) assert(FALSE);

	lpOptionValue=pCmdLine->GetMultiOption(std_string(CMDLINE_PREFIX).append(tcParam).c_str(), iParamInstance);
	if (lpOptionValue != NULL) {
		strBuf1 = ExpandEnvironmentEx(std_string(lpOptionValue));
		if (!pUtils->ValidatePath(strBuf1.c_str(), KPVPF_REGULAR_NAME | KPVPF_DEFAULT)) {
			displayIgnoredMessage(DBBckp_S->g_pAPI->GetMainWindowHandle(), 
				pCmdLine->GetTokenRel(std_string(CMDLINE_PREFIX).append(tcParam).c_str(), 0), 
				_T("ignored because destination is invalid"));
			strBuf1.clear();
		}
	}
	VERIFY(pCmdLine->Release());
	VERIFY(pUtils->Release());
	return strBuf1;
}